@@ -8,7 +8,6 @@ import configobj
 import zope.interface
 
 from acme import challenges
-
 from certbot import errors
 from certbot import interfaces
 from certbot.compat import filesystem
@@ -38,13 +37,13 @@ class DNSAuthenticator(common.Plugin):
             help='The number of seconds to wait for DNS to propagate before asking the ACME server '
                  'to verify the DNS record.')
 
-    def get_chall_pref(self, unused_domain):  # pylint: disable=missing-docstring,no-self-use
+    def get_chall_pref(self, unused_domain):  # pylint: disable=missing-function-docstring
         return [challenges.DNS01]
 
-    def prepare(self): # pylint: disable=missing-docstring
+    def prepare(self): # pylint: disable=missing-function-docstring
         pass
 
-    def perform(self, achalls): # pylint: disable=missing-docstring
+    def perform(self, achalls): # pylint: disable=missing-function-docstring
         self._setup_credentials()
 
         self._attempt_cleanup = True
@@ -67,7 +66,7 @@ class DNSAuthenticator(common.Plugin):
 
         return responses
 
-    def cleanup(self, achalls):  # pylint: disable=missing-docstring
+    def cleanup(self, achalls):  # pylint: disable=missing-function-docstring
         if self._attempt_cleanup:
             for achall in achalls:
                 domain = achall.domain
@@ -198,8 +197,7 @@ class DNSAuthenticator(common.Plugin):
 
         if code == display_util.OK:
             return response
-        else:
-            raise errors.PluginError('{0} required to proceed.'.format(label))
+        raise errors.PluginError('{0} required to proceed.'.format(label))
 
     @staticmethod
     def _prompt_for_file(label, validator=None):
@@ -232,8 +230,7 @@ class DNSAuthenticator(common.Plugin):
 
         if code == display_util.OK:
             return response
-        else:
-            raise errors.PluginError('{0} required to proceed.'.format(label))
+        raise errors.PluginError('{0} required to proceed.'.format(label))
 
 
 class CredentialsConfiguration(object):
